@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -21,6 +21,7 @@ from ._chemical import Chemical
 from ._chemicals import Chemicals, CompiledChemicals
 from ._thermal_condition import ThermalCondition
 from . import mixture
+from .mixture import Mixture
 from ._thermo import Thermo
 from ._settings import settings
 from ._thermo_data import ThermoData
@@ -34,7 +35,7 @@ from .base import functor
 from .reaction import *
 from flexsolve import speed_up
 
-__all__ = ('Chemical', 'Chemicals', 'CompiledChemicals', 'Thermo', 'Stream',
+__all__ = ('Chemical', 'Chemicals', 'CompiledChemicals', 'Mixture', 'Thermo', 'Stream',
            'MultiStream', 'ThermalCondition', 'mixture', 'ThermoData',
            *reaction.__all__, 'indexer', 'settings', 'functor', 'functors', 
            'chemicals', 'base', 'equilibrium', 'units_of_measure', 'exceptions',
@@ -51,4 +52,4 @@ pd.set_option('display.max_columns', 10)
 pd.set_option('max_colwidth', 35)
 del np, pd
 
-__version__ = "0.22.9"
\ No newline at end of file
+__version__ = "0.23.12"
\ No newline at end of file
