@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -66,6 +66,8 @@ from .units_of_measure import chemical_units_of_measure
 from .eos import GCEOS_DUMMY, PR
 from .utils import copy_maybe, check_valid_ID
 from . import functional as fn 
+from ._phase import check_phase
+from . import units_of_measure as thermo_units
 
 # from .solubility import SolubilityParameter
 # from .lennard_jones import Stockmayer, MolecularDiameter
@@ -79,7 +81,7 @@ __all__ = ('Chemical',)
 
 def get_chemical_data(chemical):
     getfield = getattr
-    return {i:getfield(chemical, i) for i in chemical.__slots__}
+    return {i:getfield(chemical, i, None) for i in chemical.__slots__}
 
 def unpickle_chemical(chemical_data):
     chemical = object.__new__(Chemical)
@@ -317,10 +319,10 @@ class Chemical:
         
     >>> # Vapor pressure (Pa)
     >>> Water.Psat(T=373.15)
-    101284.55...
+    101284.55
     >>> # Surface tension (N/m)
     >>> Water.sigma(T=298.15)
-    0.07197220523...
+    0.0719722
     >>> # Molar volume (m^3/mol)
     >>> Water.V(phase='l', T=298.15, P=101325)
     1.80692...e-05
@@ -374,7 +376,7 @@ class Chemical:
     When called, the model handle searches through each model until it finds one with an applicable domain. If none are applicable, a value error is raised:
         
     >>> Water.Psat(373.15)
-    101284.55179999319
+    101284.55
     >>> # Water.Psat(1000.0) ->
     >>> # ValueError: Water has no valid saturated vapor pressure model at T=1000.00 K
     
@@ -527,7 +529,8 @@ class Chemical:
     __slots__ = ('_ID', '_locked_state', 
                  '_phase_ref', '_eos', '_eos_1atm',
                  *_names, *_groups, 
-                 *_handles, *_data)
+                 *_handles, *_data,
+                 '_N_solutes')
     
     #: [float] Reference temperature in Kelvin
     T_ref = 298.15
@@ -551,14 +554,12 @@ class Chemical:
         chemical_cache = cls.chemical_cache
         if (cache or cls.cache) and ID in chemical_cache:
             if any([search_ID, eos, phase_ref, CAS, default, phase, 
-                    V, Cn, mu, Cp, rho, sigma, kappa, epsilon, Psat, Hvap, data]):
+                    V, Cn, mu, Cp, rho, sigma, kappa, epsilon, Psat, Hvap,
+                    data]):
                 warn('cached chemical returned; additional parameters disregarded')
             return chemical_cache[ID]
         search_ID = search_ID or ID
         if not eos: eos = PR
-        if phase: 
-            phase = phase[0].lower()
-            assert phase in ('s', 'l', 'g'), "phase must be either 's', 'l', or 'g'"
         if search_db:
             metadata = pubchem_db.search(search_ID)
             data['metadata'] = metadata
@@ -600,7 +601,6 @@ class Chemical:
         """Create a new chemical from data without searching through
         the data base, and load all possible models from given data."""
         self = super().__new__(cls)
-        check_valid_ID(ID)
         self._ID = ID
         self.reset(CAS, eos, phase_ref, phase=phase, **data)
         return self
@@ -675,6 +675,8 @@ class Chemical:
         for i in _data: setfield(self, i, None)
         for i in _energy_handles: setfield(self, i, None)
         if phase:
+            phase = phase[0]
+            check_phase(phase)
             for i in ('kappa', 'mu', 'V'):
                 setfield(self, '_' + i, TPDependentModelHandle(i))
             self._Cn = TDependentModelHandle('Cn')
@@ -704,6 +706,10 @@ class Chemical:
         
         Examples
         --------
+        >>> import thermosteam as tmo
+        >>> Glucose = tmo.Chemical('Glucose')
+        >>> Mannose = Glucose.copy('Mannose')
+        >>> Mannose.show()
         Chemical: Mannose (phase_ref='l')
         [Names]  CAS: Mannose
                  InChI: C6H12O6/c7-1-3(9)5(1...
@@ -742,7 +748,7 @@ class Chemical:
         getfield = getattr
         setfield = setattr
         for field in self.__slots__: 
-            value = getfield(self, field)
+            value = getfield(self, field, None)
             setfield(new, field, copy_maybe(value))
         new._ID = ID
         new._CAS = CAS or ID
@@ -769,6 +775,114 @@ class Chemical:
     def __reduce__(self):
         return unpickle_chemical, (get_chemical_data(self),)
     
+    ### Helpful functionality ###
+    def rho(self, *args, **kwargs):
+        """
+        Return density given thermal condition [kg/m^3].
+        
+        Examples
+        --------
+        >>> import thermosteam as tmo
+        >>> Water = tmo.Chemical('Water', cache=True)
+        >>> Water.rho('l', 298.15, 101325)
+        997.015
+        
+        """
+        return fn.V_to_rho(self.V(*args, **kwargs), self.MW)
+    
+    def Cp(self, *args, **kwargs):
+        """
+        Return heat capacity given thermal condition [J/g/K].
+        
+        Examples
+        --------
+        >>> import thermosteam as tmo
+        >>> Water = tmo.Chemical('Water', cache=True)
+        >>> Water.Cp('l', 298.15, 101325)
+        4.180
+        
+        """
+        return self.Cn(*args, **kwargs) / self.MW
+    
+    def alpha(self, *args, **kwargs):
+        """
+        Return thermal diffusivity  given thermal condition [m^2/s].
+        
+        Examples
+        --------
+        >>> import thermosteam as tmo
+        >>> Water = tmo.Chemical('Water', cache=True)
+        >>> Water.alpha('l', 298.15, 101325)
+        1.455...e-07
+        
+        """
+        return fn.alpha(self.kappa(*args, **kwargs), 
+                        self.rho(*args, **kwargs), 
+                        self.Cp(*args, **kwargs) * 1000.)
+    
+    def nu(self, *args, **kwargs):
+        """
+        Return kinematic viscosity given thermal condition [m^2/s].
+        
+        Examples
+        --------
+        >>> import thermosteam as tmo
+        >>> Water = tmo.Chemical('Water', cache=True)
+        >>> Water.nu('l', 298.15, 101325)
+        8.958...e-07
+        
+        """
+        return fn.mu_to_nu(self.mu(*args, **kwargs), 
+                           self.rho(*args, **kwargs))
+    
+    def Pr(self, *args, **kwargs):
+        """
+        Return the Prandtl number given thermal condition [-].
+        
+        Examples
+        --------
+        >>> import thermosteam as tmo
+        >>> Water = tmo.Chemical('Water', cache=True)
+        >>> Water.Pr('l', 298.15, 101325)
+        6.156
+        
+        """
+        return fn.Pr(self.Cp(*args, **kwargs) * 1000,
+                     self.kappa(*args, **kwargs), 
+                     self.mu(*args, **kwargs))
+    
+    def get_property(self, name, units, *args, **kwargs):
+        """
+        Return property in requested units.
+
+        Parameters
+        ----------
+        name : str
+            Name of stream property.
+        units : str
+            Units of measure.
+        *args, **kwargs :
+            Thermal condition.
+
+        Examples
+        --------
+        >>> import thermosteam as tmo
+        >>> Water = tmo.Chemical('Water', cache=True)
+        >>> Water.get_property('sigma', 'N/m', 300.) # Surface tension
+        0.071685
+
+        >>> Water.get_property('rho', 'g/cm3', 'l', 300., 101325) # Density
+        0.996267
+
+        """
+        value = getattr(self, name)(*args, **kwargs)
+        units_dct = thermo_units.chemical_units_of_measure
+        if name in units_dct:
+            original_units = units_dct[name]
+        else:
+            raise ValueError(f"'{name}' is not thermodynamic property")
+        return original_units.convert(value, units)
+    
     @property
     def phase_ref(self):
         """{'s', 'l', 'g'} Phase at 298 K and 101325 Pa."""
@@ -1703,7 +1817,7 @@ class Chemical:
                 mu.add_model(0.00091272)
         if 'V' in properties:
             V = self._V
-            V_default = fn.rho_to_V(1050, MW)
+            V_default = fn.rho_to_V(1000, MW)
             if hasfield(V, 'l'):
                 V.l.add_model(V_default)
             elif not V:
@@ -1834,6 +1948,14 @@ class Chemical:
         """[str] Constant phase of chemical."""
         return self._locked_state
     
+    @property
+    def N_solutes(self):
+        """[int] Number of molecules formed when solvated."""
+        return getattr(self, '_N_solutes', None)
+    @N_solutes.setter
+    def N_solutes(self, N_solutes):
+        self._N_solutes = int(N_solutes)
+    
     def at_state(self, phase, copy=False):
         """
         Set the state of chemical.
@@ -1896,6 +2018,8 @@ class Chemical:
         return f"Chemical('{self}')"
     
 def lock_phase(chemical, phase):
+    phase = phase[0]
+    check_phase(phase)
     getfield = getattr
     setfield = object.__setattr__
     hasfield = hasattr
