@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -23,9 +23,9 @@ def must_compile(*args, **kwargs): # pragma: no cover
     raise TypeError("method valid only for compiled chemicals; "
                     "run <Chemicals>.compile() to compile")
 
-def chemical_data_array(chemicals, attr):
+def chemical_data_array(chemicals, attr, dtype=float):
     getfield = getattr
-    data = np.asarray([getfield(i, attr) for i in chemicals], dtype=float)
+    data = np.asarray([getfield(i, attr) for i in chemicals], dtype)
     data.setflags(0)
     return data
     
@@ -241,8 +241,12 @@ class Chemicals:
         volume model at T=298.15 K and P=101325 Pa
         
         """
-        CompiledChemicals._compile(self, skip_checks)
         setattr(self, '__class__', CompiledChemicals)
+        try: self._compile(skip_checks)
+        except Exception as error:
+            setattr(self, '__class__', Chemicals)
+            setattr(self, '__dict__', Chemicals(self).__dict__)
+            raise error
     
     kwarray = array = index = indices = must_compile
         
@@ -371,7 +375,7 @@ class CompiledChemicals(Chemicals):
     def __reduce__(self):
         return CompiledChemicals, (self.tuple,)
     
-    def compile(self):
+    def compile(self, skip_checks=False):
         """Do nothing, CompiledChemicals objects are already compiled.""" 
     
     def refresh_constants(self):
@@ -439,6 +443,23 @@ class CompiledChemicals(Chemicals):
         dct['_index'] = index = dict((*zip(CAS, index),
                                       *zip(IDs, index)))
         dct['_index_cache'] = {}
+        repeated_names = set()
+        names = set()
+        for i in chemicals:
+            if not i.iupac_name: i.iupac_name = ()
+            all_names = (*i.iupac_name, i.common_name, i.formula)
+            for name in all_names:
+                if not name: continue
+                if name in names:
+                    repeated_names.add(name)
+                else:
+                    names.add(name)
+        for i in chemicals:
+            all_names = (*i.iupac_name, i.common_name, i.formula)
+            ID = i.ID
+            for name in all_names:
+                if name and name not in repeated_names:
+                    self.set_synonym(ID, name)
         vle_chemicals = []
         lle_chemicals = []
         heavy_chemicals = []
@@ -448,8 +469,9 @@ class CompiledChemicals(Chemicals):
             if locked_phase:
                 if locked_phase in ('s', 'l'):
                     heavy_chemicals.append(i)
-                    if i.Dortmund or i.UNIFAC:
+                    if i.Dortmund or i.UNIFAC or i.NIST or i.PSRK:
                         lle_chemicals.append(i)
+                    if i.N_solutes is None: i._N_solutes = 0
                 elif locked_phase == 'g':
                     light_chemicals.append(i)
                 else:
@@ -463,6 +485,7 @@ class CompiledChemicals(Chemicals):
         dct['light_chemicals'] = tuple_(light_chemicals)
         dct['_has_vle'] = has_vle = np.zeros(size, dtype=bool)
         dct['_has_lle'] = has_lle = np.zeros(size, dtype=bool)
+        dct['_heavy_solutes'] = chemical_data_array(heavy_chemicals, 'N_solutes')
         dct['_heavy_indices'] = [index[i.ID] for i in heavy_chemicals]
         dct['_light_indices'] = [index[i.ID] for i in light_chemicals]
         vle_index = [index[i.ID] for i in vle_chemicals]
@@ -634,6 +657,30 @@ class CompiledChemicals(Chemicals):
                 except: pass
         return new
     
+    def get_parsable_synonym(self, ID):
+        """
+        Return a synonym of the given chemical identifier that can be 
+        parsed by Python as a variable name
+        
+        Parameters
+        ----------
+        ID : str
+            Chemical identifier.
+            
+        Examples
+        --------
+        Get parsable synonym of 2,3-Butanediol:
+        
+        >>> from thermosteam import CompiledChemicals
+        >>> chemicals = CompiledChemicals(['2,3-Butanediol'], cache=True)
+        >>> chemicals.get_parsable_synonym('2,3-Butanediol')
+        'C4H10O2'
+        
+        """
+        isvalid = utils.is_valid_ID      
+        for i in self.get_synonyms(ID):
+            if isvalid(i): return i
+    
     def get_synonyms(self, ID):
         """
         Get all synonyms of a chemical.
@@ -650,8 +697,7 @@ class CompiledChemicals(Chemicals):
         >>> from thermosteam import CompiledChemicals
         >>> chemicals = CompiledChemicals(['Water'], cache=True)
         >>> chemicals.get_synonyms('Water')
-        ['7732-18-5', 'Water']
-        
+        ['7732-18-5', 'Water', 'oxidane', 'water', 'H2O']
         
         """
         k = self._index[ID]
