@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -10,7 +10,6 @@
 from ._stream import Stream
 from ._thermal_condition import ThermalCondition
 from .indexer import MolarFlowIndexer
-from ._settings import settings
 from ._phase import phase_tuple
 from . import equilibrium as eq
 from . import utils
@@ -29,12 +28,6 @@ def get_phase_fraction(stream, phases):
             phase_fraction += imol[phase].sum() 
     return phase_fraction / F_mol
 
-def phase_is_present(stream, phases):
-    all_phases = stream.phases
-    imol = stream.imol
-    for phase in phases:
-        if phase in all_phases and imol[phase].any(): return True
-    return False
 
 class MultiStream(Stream):
     """
@@ -255,6 +248,47 @@ class MultiStream(Stream):
     def __iter__(self):
         for i in self.phases: yield self[i]
     
+    def __len__(self):
+        return len(self.phases)
+    
+    def shares_flow_rate_with(self, other):
+        """
+        Return whether other stream shares data with this one.
+        
+        Examples
+        --------
+        >>> import thermosteam as tmo
+        >>> tmo.settings.set_thermo(['Water'], cache=True)
+        >>> s1 = tmo.MultiStream('s1')
+        >>> other = s1.flow_proxy()
+        >>> s1.shares_flow_rate_with(other)
+        True
+        >>> s1 = tmo.MultiStream('s1', phases=('l', 'g'))
+        >>> s1.shares_flow_rate_with(s1['g'])
+        True
+        >>> s2 = tmo.MultiStream('s2', phases=('l', 'g'))
+        >>> s2.shares_flow_rate_with(s1['g'])
+        False
+        >>> s1.shares_flow_rate_with(s2)
+        False
+        
+        """
+        imol = self._imol
+        other_imol = other._imol
+        if imol.__class__ is other_imol.__class__ and imol._data is other_imol._data:
+            shares_data = True
+        elif isinstance(other, Stream):
+            phase = other.phase
+            substreams = self._streams
+            if phase in substreams:
+                substream = substreams[phase]
+                shares_data = other.shares_flow_rate_with(substream)
+            else:
+                shares_data = False
+        else:
+            shares_data = False
+        return shares_data
+    
     ### Property getters ###
     
     def get_flow(self, units, key=...):
@@ -411,7 +445,6 @@ class MultiStream(Stream):
     def mu(self):
         """[float] Hydrolic viscosity [Pa*s]."""
         return self.mixture.xmu(self._imol.iter_composition(), *self._thermal_condition)
-
     @property
     def sigma(self):
         """[float] Surface tension [N/m]."""
@@ -422,6 +455,26 @@ class MultiStream(Stream):
         """[float] Relative permittivity [-]."""
         mol = self._imol['l']
         return self.mixture.epsilon(mol / mol.sum(), *self._thermal_condition)
+    @property
+    def Cp(self):
+        """[float] Heat capacity [J/g/K]."""
+        return self.mixture.xCp(self._imol.iter_composition(), self.T)
+    @property
+    def alpha(self):
+        """[float] Thermal diffusivity [m^2/s]."""
+        return self.mixture.xalpha(self._imol.iter_composition(), *self._thermal_condition)
+    @property
+    def rho(self):
+        """[float] Density [kg/m^3]."""
+        return self.mixture.xrho(self._imol.iter_composition(), *self._thermal_condition)
+    @property
+    def nu(self):
+        """[float] Kinematic viscosity [m^2/s]."""
+        return self.mixture.xnu(self._imol.iter_composition(), *self._thermal_condition)
+    @property
+    def Pr(self):
+        """[float] Prandtl number [-]."""
+        return self.mixture.xPr(self._imol.iter_composition(), *self._thermal_condition)
         
     ### Methods ###
     
@@ -567,25 +620,21 @@ class MultiStream(Stream):
                 other_phase_index = self.imol.get_phase_index(other.phase)
                 data[other_phase_index, :] = other_data
                 data[phase_index, IDs_index] = original_data[phase_index, IDs_index]
-                if remove:
-                    if phase is ... or phase_index == other_phase_index:
-                        excluded_data = other_data[IDs_index]
-                        other_data[:] = 0.
-                        other_data[IDs_index] = excluded_data   
+                if remove and (phase is ... or phase_index == other_phase_index):
+                    excluded_data = other_data[IDs_index]
+                    other_data[:] = 0.
+                    other_data[IDs_index] = excluded_data   
+        elif isinstance(other, MultiStream):
+            if self.phases != other.phases:
+                raise ValueError('other stream must have the same phases defined to copy flow')
+            data[phase_index, IDs_index] = other_data[phase_index, IDs_index]
+            if remove: other_data[phase_index, IDs_index] = 0.
         else:
-            if isinstance(other, MultiStream):
-                if self.phases != other.phases:
-                    raise ValueError('other stream must have the same phases defined to copy flow')
-                data[phase_index, IDs_index] = other_data[phase_index, IDs_index]
-                if remove:
-                    other_data[phase_index, IDs_index] = 0.
-            else:
-                data[:] = 0.
-                other_phase_index = self.imol.get_phase_index(other.phase)
-                if phase is ... or phase_index == other_phase_index:
-                    data[other_phase_index, IDs_index] = other_data[IDs_index]
-                    if remove:
-                        other_data[IDs_index] = 0.
+            data[:] = 0.
+            other_phase_index = self.imol.get_phase_index(other.phase)
+            if phase is ... or phase_index == other_phase_index:
+                data[other_phase_index, IDs_index] = other_data[IDs_index]
+                if remove: other_data[IDs_index] = 0.
     
     def get_normalized_mol(self, IDs):
         """
@@ -763,21 +812,24 @@ class MultiStream(Stream):
     
     @property
     def phase(self):
-        return ''.join([phases[0] for phases in ('gG', 'lL', 'sS') if phase_is_present(self, phases)])
+        imol = self._imol
+        return ''.join([phases[0] for phases in ('gG', 'lL', 'sS') if not imol.phases_are_empty(phases)])
     @phase.setter
     def phase(self, phase):
+        if len(phase) > 1: self.phases = phase
         if self._link: raise RuntimeError('cannot convert linked stream')
         self._imol = self._imol.to_chemical_indexer(phase)
+        self._streams.clear()
         self.__class__ = Stream
     
     ### Representation ###
     
-    def _info(self, T, P, flow, composition, N):
+    def _info(self, T, P, flow, composition, N, IDs):
         """Return string with all specifications."""
         from .indexer import nonzeros
-        IDs = self.chemicals.IDs
+        if not IDs: IDs = self.chemicals.IDs
         basic_info = self._basic_info()
-        all_IDs, _ = nonzeros(self.chemicals.IDs, self.mol)
+        all_IDs, _ = nonzeros(IDs, self.imol[IDs])
         all_IDs = tuple(all_IDs)
         display_units = self.display_units
         T_units = T or display_units.T
