@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -11,7 +11,6 @@ import numpy as np
 import thermosteam as tmo
 from . import indexer
 from . import equilibrium as eq
-from . import functional as fn
 from . import units_of_measure as thermo_units
 from .exceptions import DimensionError
 from chemicals.elements import array_to_atoms, symbol_to_index
@@ -25,6 +24,16 @@ mol_units = indexer.ChemicalMolarFlowIndexer.units
 mass_units = indexer.ChemicalMassFlowIndexer.units
 vol_units = indexer.ChemicalVolumetricFlowIndexer.units
 
+class StreamData:
+    __slots__ = ('_imol', '_T', '_P', '_phases')
+    
+    def __init__(self, imol, thermal_condition, phases):
+        self._imol = imol.copy()
+        self._T = thermal_condition._T
+        self._P = thermal_condition._P
+        self._phases = phases
+        
+    
 # %%
 
 @utils.thermo_user
@@ -249,9 +258,104 @@ class Stream:
         self._register(ID)
         self._link = None
 
+    def shares_flow_rate_with(self, other):
+        """
+        Return whether other stream shares data with this one.
+        
+        Examples
+        --------
+        >>> import thermosteam as tmo
+        >>> tmo.settings.set_thermo(['Water'], cache=True)
+        >>> s1 = tmo.Stream('s1')
+        >>> other = s1.flow_proxy()
+        >>> s1.shares_flow_rate_with(other)
+        True
+        >>> s1 = tmo.MultiStream('s1', phases=('l', 'g'))
+        >>> s1['g'].shares_flow_rate_with(s1)
+        True
+        >>> s2 = tmo.MultiStream('s2', phases=('l', 'g'))
+        >>> s1['g'].shares_flow_rate_with(s2)
+        False
+        >>> s1['g'].shares_flow_rate_with(s2['g'])
+        False
+        
+        """
+        imol = self._imol
+        other_imol = other._imol
+        if imol.__class__ is other_imol.__class__ and imol._data is other_imol._data:
+            shares_data = True
+        elif isinstance(other, tmo.MultiStream):
+            phase = self.phase
+            substreams = other._streams
+            if phase in substreams:
+                substream = substreams[phase]
+                shares_data = self.shares_flow_rate_with(substream)
+            else:
+                shares_data = False
+        else:
+            shares_data = False
+        return shares_data
+
     def as_stream(self):
         """Does nothing."""
 
+    def get_data(self):
+        """
+        Return a StreamData object containing data on material flow rates, 
+        temperature, pressure, and phase(s).
+        
+        See Also
+        --------
+        Stream.set_data
+        
+        Examples
+        --------
+        Get and set data from stream at different conditions
+        
+        >>> import thermosteam as tmo
+        >>> tmo.settings.set_thermo(['Water'], cache=True)
+        >>> stream = tmo.Stream('stream', Water=10)
+        >>> data = stream.get_data()
+        >>> stream.vle(V=0.5, P=101325)
+        >>> data_vle = stream.get_data()
+        >>> stream.set_data(data)
+        >>> stream.show()
+        Stream: stream
+         phase: 'l', T: 298.15 K, P: 101325 Pa
+         flow (kmol/hr): Water  10
+        >>> stream.set_data(data_vle)
+        >>> stream.show()
+        MultiStream: stream
+         phases: ('g', 'l'), T: 373.12 K, P: 101325 Pa
+         flow (kmol/hr): (g) Water  5
+                         (l) Water  5
+        
+        Note that only StreamData objects are valid for this method:
+        
+        >>> stream.set_data({'T': 298.15})
+        Traceback (most recent call last):
+        ValueError: stream_data must be a StreamData object; not dict
+        
+        """
+        return StreamData(self._imol, self._thermal_condition, self.phases)
+
+    def set_data(self, stream_data):
+        """
+        Set material flow rates, temperature, pressure, and phase(s) through a 
+        StreamData object
+
+        See Also
+        --------
+        Stream.get_data
+
+        """
+        if isinstance(stream_data, StreamData):
+            self.phases = stream_data._phases
+            self._imol.copy_like(stream_data._imol)
+            self._thermal_condition.copy_like(stream_data)
+        else:
+            raise ValueError(f'stream_data must be a StreamData object; not {type(stream_data).__name__}')
+        
     @property
     def price(self):
         """[float] Price of stream per unit mass [USD/kg."""
@@ -273,7 +377,7 @@ class Stream:
         True
         
         """
-        return (self._imol._data == 0.).all()
+        return self._imol.isempty()
 
     @property
     def vapor_fraction(self):
@@ -674,7 +778,20 @@ class Stream:
         return self.mixture.H(self.phase, self.mol, *self._thermal_condition)
     @H.setter
     def H(self, H:float):
-        self.T = self.mixture.solve_T(self.phase, self.mol, H, *self._thermal_condition)
+        try: self.T = self.mixture.solve_T(self.phase, self.mol, H,
+                                           *self._thermal_condition)
+        except Exception as error: # pragma: no cover
+            phase = self.phase.lower()
+            if phase == 'g':
+                # Maybe too little heat, liquid must be present
+                self.phase = 'l'
+            elif phase == 'l':
+                # Maybe too much heat, gas must be present
+                self.phase = 'g'
+            else:
+                raise error
+            self.T = self.mixture.solve_T(self.phase, self.mol, H,
+                                          *self._thermal_condition)
 
     @property
     def S(self):
@@ -701,7 +818,9 @@ class Stream:
     @property
     def Hvap(self):
         """[float] Enthalpy of vaporization flow rate in kJ/hr."""
-        return self.mixture.Hvap(self.mol, *self._thermal_condition)
+        mol = self.mol
+        condition = self._thermal_condition
+        return sum([i*j.Hvap(condition.T) for i,j in zip(mol, self.chemicals) if i and not j.locked_state])
     
     @property
     def C(self):
@@ -735,58 +854,61 @@ class Stream:
     @property
     def MW(self):
         """[float] Overall molecular weight."""
-        return self.F_mass / self.F_mol
+        return self.mixture.MW(self.mol)
     @property
     def V(self):
         """[float] Molar volume [m^3/mol]."""
-        mol = self.mol
-        return self.mixture.V(self.phase, mol / mol.sum(), *self._thermal_condition)
+        return self.mixture.V(*self._imol.get_phase_and_composition(),
+                              *self._thermal_condition)
     @property
     def kappa(self):
         """[float] Thermal conductivity [W/m/k]."""
-        mol = self.mol
-        return self.mixture.kappa(self.phase, mol / mol.sum(), *self._thermal_condition)
+        return self.mixture.kappa(*self._imol.get_phase_and_composition(),
+                                  *self._thermal_condition)
     @property
     def Cn(self):
         """[float] Molar heat capacity [J/mol/K]."""
-        mol = self.mol
-        return self.mixture.Cn(self.phase, mol / mol.sum(), self.T)
+        return self.mixture.Cn(*self._imol.get_phase_and_composition(), self.T)
     @property
     def mu(self):
         """[float] Hydrolic viscosity [Pa*s]."""
-        mol = self.mol
-        return self.mixture.mu(self.phase, mol / mol.sum(), *self._thermal_condition)
+        return self.mixture.mu(*self._imol.get_phase_and_composition(),
+                               *self._thermal_condition)
     @property
     def sigma(self):
         """[float] Surface tension [N/m]."""
         mol = self.mol
-        return self.mixture.sigma(mol / mol.sum(), *self._thermal_condition)
+        return self.mixture.sigma(mol / mol.sum(),
+                                  *self._thermal_condition)
     @property
     def epsilon(self):
         """[float] Relative permittivity [-]."""
         mol = self.mol
         return self.mixture.epsilon(mol / mol.sum(), *self._thermal_condition)
-    
     @property
     def Cp(self):
         """[float] Heat capacity [J/g/K]."""
-        return self.Cn / self.MW
+        return self.mixture.Cp(*self._imol.get_phase_and_composition(), self.T)
     @property
     def alpha(self):
         """[float] Thermal diffusivity [m^2/s]."""
-        return fn.alpha(self.kappa, self.rho, self.Cp * 1000.)
+        return self.mixture.alpha(*self._imol.get_phase_and_composition(),
+                                  *self._thermal_condition)
     @property
     def rho(self):
         """[float] Density [kg/m^3]."""
-        return fn.V_to_rho(self.V, self.MW)
+        return self.mixture.rho(*self._imol.get_phase_and_composition(), 
+                                *self._thermal_condition)
     @property
     def nu(self):
-        """[float] Kinematic viscosity [-]."""
-        return fn.mu_to_nu(self.mu, self.rho)
+        """[float] Kinematic viscosity [m^2/s]."""
+        return self.mixture.nu(*self._imol.get_phase_and_composition(),
+                               *self._thermal_condition)
     @property
     def Pr(self):
         """[float] Prandtl number [-]."""
-        return fn.Pr(self.Cp * 1000, self.kappa, self.mu)
+        return self.mixture.Pr(*self._imol.get_phase_and_composition(),
+                               *self._thermal_condition)
     
     ### Stream methods ###
     
@@ -848,7 +970,7 @@ class Stream:
             new = cls(ID, thermo=thermo)
         return new
     
-    def mix_from(self, others):
+    def mix_from(self, others, energy_balance=True):
         """
         Mix all other streams into this one, ignoring its initial contents.
         
@@ -884,6 +1006,14 @@ class Stream:
          flow (kg/hr): Water    40
                        Ethanol  20
         
+        Mixing emptry streams if fine too:
+            
+        >>> s1.empty(); s2.empty(); s_mix.mix_from([s1, s2])
+        >>> s_mix.show()
+        Stream: s_mix
+         phase: 'l', T: 298.15 K, P: 101325 Pa
+         flow: 0
+        
         """
         others = [i for i in others if i]
         N_others = len(others)
@@ -893,31 +1023,16 @@ class Stream:
             self.copy_like(others[0])
         else:
             self._imol.mix_from([i._imol for i in others])
-            H = sum([i.H for i in others])
-            if self.isempty():
-                self.T = np.mean([i.T for i in others])
-            try: 
-                self.H = H
-            except: # pragma: no cover
-                phase = self.phase.lower()
-                if phase == 'g':
-                    # Maybe too little heat, liquid must be present
-                    self.phase = 'l'
-                elif phase == 'l':
-                    # Maybe too much heat, gas must be present
-                    self.phase = 'g'
-                else:
-                    phases = ''.join([i.phase for i in others])
-                    self.phases = tuple(set(phases))
-                    self._imol.mix_from([i._imol for i in others])
-                try: 
+            if energy_balance and not self.isempty():
+                H = sum([i.H for i in others])
+                try:
                     self.H = H
                 except:
                     phases = ''.join([i.phase for i in others])
                     self.phases = tuple(set(phases))
                     self._imol.mix_from([i._imol for i in others])
                     self.H = H
-            
+                
     def split_to(self, s1, s2, split):
         """
         Split molar flow rate from this stream to two others given
@@ -1018,11 +1133,21 @@ class Stream:
         >>> s2.mol is s1.mol
         False
         
+        MultiStream phases cannot be unlinked:
+        
+        >>> s1 = tmo.MultiStream(None, phases=('l', 'g'))
+        >>> s1['g'].unlink()
+        Traceback (most recent call last):
+        RuntimeError: phase is locked; stream cannot be unlinked
+        
         """
-        self._imol._data_cache.clear()
+        imol = self._imol
+        if hasattr(imol, '_phase') and isinstance(imol._phase, tmo._phase.LockedPhase):
+            raise RuntimeError('phase is locked; stream cannot be unlinked')
+        imol._data_cache.clear()
+        imol._data = imol._data.copy()
+        imol._phase = imol._phase.copy()
         self._thermal_condition = self._thermal_condition.copy()
-        self._imol._data = self._imol._data.copy()
-        self._imol._phase = self._imol._phase.copy()
         self._init_cache()
         self._link = None
     
@@ -1630,11 +1755,6 @@ class Stream:
         """
         Receive vapors from another stream as if in equilibrium.
 
-        Notes
-        -----
-        Disregards energy balance. Assumes liquid composition will not change
-        significantly.
-
         Examples
         --------
         >>> import thermosteam as tmo
@@ -1650,7 +1770,7 @@ class Stream:
                          Ethanol  0.0616
                          N2       0.369
         """
-        # TODO: Add energy balance 
+        ms = tmo.MultiStream(None)
         assert self.phase == 'g', 'stream must be a gas to receive vent'
         chemicals = other.vle_chemicals
         light_indices = other.chemicals._light_indices
@@ -1677,6 +1797,8 @@ class Stream:
         index = other.mol < 0.
         self.mol[index] += other.mol[index]
         other.mol[index] = 0
+        ms.mix_from([self, other])
+        self.T = other.T = ms.T
         
     ### Casting ###
     
@@ -1731,12 +1853,19 @@ class Stream:
         s = '' if isinstance(phase, str) else 's'
         return f" phase{s}: {repr(phase)}, T: {T:.5g} {T_units}, P: {P:.6g} {P_units}\n"
     
-    def _info(self, T, P, flow, composition, N):
+    def _source_info(self):
+        source = self.source
+        return f"{source}-{source.outs.index(self)}" if source else self.ID
+    
+    def _info(self, T, P, flow, composition, N, IDs):
         """Return string with all specifications."""
         from .indexer import nonzeros
         basic_info = self._basic_info()
-        IDs = self.chemicals.IDs
-        data = self.imol.data
+        if not IDs:
+            IDs = self.chemicals.IDs
+            data = self.imol.data
+        else:
+            data = self.imol[IDs]
         IDs, data = nonzeros(IDs, data)
         IDs = tuple(IDs)
         display_units = self.display_units
@@ -1744,8 +1873,10 @@ class Stream:
         P_units = P or display_units.P
         flow_units = flow or display_units.flow
         N_max = display_units.N if N is None else N
-        composition = display_units.composition if composition is None else composition 
         basic_info += self._info_phaseTP(self.phase, T_units, P_units)
+        if N_max == 0:
+            return basic_info[:-1]
+        composition = display_units.composition if composition is None else composition 
         N_IDs = len(IDs)
         if N_IDs == 0:
             return basic_info + ' flow: 0' 
@@ -1781,28 +1912,30 @@ class Stream:
               + beginning
               + flow_rates)
 
-    def show(self, T=None, P=None, flow=None, composition=None, N=None):
+    def show(self, T=None, P=None, flow=None, composition=None, N=None, IDs=None):
         """Print all specifications.
         
         Parameters
         ----------
-        T: str, optional
+        T : str, optional
             Temperature units.
-        P: str, optional
+        P : str, optional
             Pressure units.
-        flow: str, optional
+        flow : str, optional
             Flow rate units.
-        composition: bool, optional
+        composition : bool, optional
             Whether to show composition.
-        N: int, optional
+        N : int, optional
             Number of compounds to display.
-        
+        IDs : tuple[str], optional
+            IDs of compounds to display. Defaults to all chemicals
+            .
         Notes
         -----
         Default values are stored in `Stream.display_units`.
         
         """
-        print(self._info(T, P, flow, composition, N))
+        print(self._info(T, P, flow, composition, N, IDs))
     _ipython_display_ = show
     
     def print(self, units=None):
