@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -14,6 +14,7 @@ from ..exceptions import DomainError, InfeasibleRegion
 from .solve_vle_composition import solve_x
 from ..utils import fill_like, Cache
 from .._settings import settings
+from .domain import vle_domain
 
 __all__ = ('DewPoint', 'DewPointCache')
 
@@ -84,8 +85,9 @@ class DewPoint:
             self.phi = thermo.Phi(chemicals)
             self.pcf = thermo.PCF(chemicals)
             self.Psats = Psats = [i.Psat for i in chemicals]
-            self.Tmin = Tmin = max(max([i.Tmin for i in Psats]) + 1e-3, self.Tmin_default)
-            self.Tmax = Tmax = min([i.Tmax for i in Psats]) - 1e-3
+            Tmin, Tmax = vle_domain(chemicals)
+            self.Tmin = Tmin
+            self.Tmax = Tmax
             self.Pmin = min([i(Tmin) for i in Psats])
             self.Pmax = max([i(Tmax) for i in Psats])
             self.chemicals = chemicals
@@ -165,13 +167,16 @@ class DewPoint:
         >>> DP = tmo.equilibrium.DewPoint(chemicals)
         >>> DP.solve_Tx(z=np.array([0.5, 0.5]), P=101325)
         (357.451847, array([0.849, 0.151]))
+        
         """
+        if P > self.Pmax: P = self.Pmax
+        elif P < self.Pmin: P = self.Pmin
         f = self._T_error
         z_norm = z/z.sum()
         zP = z * P
         args = (P, z_norm, zP)
         self.P = P
-        T_guess = self.T or self._T_ideal(zP) 
+        T_guess = self._T_ideal(zP) 
         try:
             T = flx.aitken_secant(f, T_guess, T_guess + 1e-3,
                                   1e-9, 5e-12, args,
@@ -215,13 +220,15 @@ class DewPoint:
         (82444.29876, array([0.853, 0.147]))
  
        """
+        if T > self.Tmax: T = self.Tmax
+        elif T < self.Tmin: T = self.Tmin
         z_norm = z/z.sum()
         Psats = array([i(T) for i in self.Psats], dtype=float)
         z_over_Psats = z/Psats
         args = (T, z_norm, z_over_Psats)
         self.T = T
         f = self._P_error
-        P_guess = self.P or self._P_ideal(z_over_Psats)
+        P_guess = self._P_ideal(z_over_Psats)
         try:
             P = flx.aitken_secant(f, P_guess, P_guess-10, 1e-3, 5e-12, args,
                                   checkiter=False)
