@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -141,7 +141,8 @@ class LLE(Equilibrium, phases='lL'):
                     MW = self.chemicals.MW[index]
                     mass_L = mol_L * MW
                     mass_l = mol_l * MW
-                    top_chemical_index = self.chemicals.index(top_chemical)
+                    IDs = {i.ID: n for n, i in enumerate(lle_chemicals)}
+                    top_chemical_index = IDs[top_chemical]
                     C_L = mass_L[top_chemical_index] / mass_L.sum()
                     C_l = mass_l[top_chemical_index] / mass_l.sum()
                     top_L = C_L > C_l
