@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -33,7 +33,8 @@ def solve_x(x_gamma_poyinting, gamma, poyinting, T, x_guess):
     if x_guess is None: x_guess = x_gamma_poyinting
     args = (x_gamma_poyinting, gamma, poyinting, T)
     try:
-        x = flx.aitken(x_iter, x_guess, 1e-10, args=args, checkiter=False)
+        x = flx.wegstein(x_iter, x_guess, 1e-10, args=args, checkiter=False,
+                         checkconvergence=False, convergenceiter=3)
     except InfeasibleRegion:
         x = x_gamma_poyinting
     return x
@@ -45,5 +46,7 @@ def y_iter(y, y_phi, phi, T, P):
 def solve_y(y_phi, phi, T, P, y_guess):
     if isinstance(phi, IdealFugacityCoefficients): return y_phi
     elif y_guess is None: y_guess = y_phi
-    return flx.aitken(y_iter, y_phi, 1e-9, args=(y_phi, phi, T, P),
-                      checkiter=False)
\ No newline at end of file
+    return flx.wegstein(y_iter, y_phi, 1e-9, args=(y_phi, phi, T, P), 
+                        checkiter=False,
+                        checkconvergence=False, 
+                        convergenceiter=3)
\ No newline at end of file
